/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is SearchWords.
 *
 * The Initial Developer of the Original Code is
 * Ryan Flint <rflint@dslr.net>.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */
//XXX Find a way to do this reliably without redefining browser functions
getShortcutOrURI = function(aURL, aPostDataRef) {
  const Cc = Components.classes;
  const Ci = Components.interfaces;
  const kCHARSET = /^(.*)\&mozcharset=([a-zA-Z][_\-a-zA-Z0-9]+)\s*$/;
  var shortcutURL = null;
  var places = false;

  var sServ = Cc["@mozilla.org/browser/search-service;1"].
                getService(Ci.nsIBrowserSearchService);
  // Are we on a Places enabled build?
  try {
    var bmServ = Cc["@mozilla.org/browser/nav-bookmarks-service;1"].
                   getService(Ci.nsINavBookmarksService);
    if (!BMSVC)
      places = true;
  } catch(e) {}

  // First attempt to resolve as a single keyword with no params
  var engine = sServ.getEngineByAlias(aURL);
  if (engine)
    shortcutURL = engine.getSubmission('', 'text/html').uri.spec;

  else if (places) {
    var shortcutURI = bmServ.getURIForKeyword(aURL);
    if (shortcutURI)
      shortcutURL = shortcutURI.spec;
  }

  else {
    try {
      shortcutURL = BMSVC.resolveKeyword(aURL, aPostDataRef);
    } catch(e) {}
  }

  // Now attempt to resolve <keyword> <foo>
  if (!shortcutURL) {
    var aOffset = aURL.indexOf(" ");
    if (aOffset > 0) {
      var cmd = aURL.substr(0, aOffset);
      var text = aURL.substr(aOffset+1);
      var ss = false;

      var engine = sServ.getEngineByAlias(cmd);
      if (engine) {
        shortcutURL = engine.getSubmission(text, 'text/html').uri.spec;
        ss = true;
      }

      else if (places) {
        shortcutURI = bookmarkService.getURIForKeyword(cmd);
        if (shortcutURI)
          shortcutURL = shortcutURI.spec;
      }

      else {
        try {
          shortcutURL = BMSVC.resolveKeyword(cmd, aPostDataRef);
        } catch(e) {}
      }

      if (shortcutURL && text && !ss) {
        var encodedText = null; 
        var charset = "";
        var matches = shortcutURL.match(kCHARSET);

        if (matches) {
          shortcutURL = matches[1];
          charset = matches[2];
        }

        //XXX Bug 317472 - No lastCharset in Places yet.
        else if (!ss && /%s/.test(shortcutURL) ||
                  (aPostDataRef && /%s/.test(aPostDataRef.value))) {
          try {
            charset = BMSVC.getLastCharset(shortcutURL);
          } catch (ex) {}
        }

        if (charset)
          encodedText = escape(convertFromUnicode(charset, text)); 
        else  // default case: charset=UTF-8
          encodedText = encodeURIComponent(text);

        if (aPostDataRef && aPostDataRef.value) {
          aPostDataRef.value = unescape(aPostDataRef.value);
          if (aPostDataRef.value.match(/%[sS]/)) {
            aPostDataRef.value = getPostDataStream(aPostDataRef.value,
                                                   text, encodedText,
                                                   "application/x-www-form-urlencoded");
          }
          else {
            shortcutURL = null;
            aPostDataRef.value = null;
          }
        }
        else {
          if (/%[sS]/.test(shortcutURL))
            shortcutURL = shortcutURL.replace(/%s/g, encodedText).replace(/%S/g, text);
          else 
            shortcutURL = null;
        }
      }
    }
  }

  if (shortcutURL)
    aURL = shortcutURL;

  return aURL;
};