/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is SearchWords.
 *
 * The Initial Developer of the Original Code is
 * Ryan Flint <rflint@dslr.net>.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Gavin Sharp <gavin@gavinsharp.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */
const kXUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

EngineView.prototype.getCellText = function(index, column) {
  if (column.id == "engineName")
    return this._engineStore.engines[index].name;
  else if (column.id == "engineAlias")
    return this._engineStore.engines[index].alias;
};

function EngineChangeOp(aEngineClone, aProp, aNewValue) {
  if (!aEngineClone || !aProp)
    throw new Error("bad args to new EngineChangeOp!");

  this._engine = aEngineClone.originalEngine;
  this._prop = aProp;
  this._newValue = aNewValue;
}

EngineChangeOp.prototype = {
  _engine: null,
  _prop: null,
  _newValue: null,
  commit: function ERO_commit() {
    this._engine[this._prop] = this._newValue;
  }
};

EngineStore.prototype.changeEngine = function ES_changeEngine(aEngine, aProp, aNewValue) {
  var index = this._getIndexForEngine(aEngine);

  if (index == -1)
    throw new Error("invalid engine?");

  this._engines[index][aProp] = aNewValue;
  this._ops.push(new EngineChangeOp(aEngine, aProp, aNewValue));
};

gEngineManagerDialog.editAlias = function engineManager_editAlias() {
  var selectedEngine = gEngineView.selectedEngine;
  if (!selectedEngine)
    return;

  var prompt = Cc["@mozilla.org/embedcomp/prompt-service;1"].
               getService(Ci.nsIPromptService);
  var alias = { value: selectedEngine.alias };

  var title = schwords.strings.getString('etitle');
  var msg = schwords.strings.getFormattedString('etext', [selectedEngine.name]);

  if (prompt.prompt(window, title, msg, alias, null, { })) {
    gEngineView._engineStore.changeEngine(selectedEngine, "alias",
                                          alias.value);
    gEngineView.invalidate();
  }
};

var schwords = {
  init: function() {
    this.strings = document.getElementById("searchwords-strings");

    var dia = document.getElementById('engineManager');
    var vboxes = dia.getElementsByTagName('vbox');
    var cvbox = null;
    for (var i = 0; i < vboxes.length; i++) {
      if (vboxes[i].hasChildNodes()) {
        for (var it = 0; it < vboxes[i].childNodes.length; it++) {
          if (vboxes[i].childNodes[it].id == 'up')
            cvbox = vboxes[i];
        }
      }
    }

    var btn = document.createElementNS(kXUL_NS, 'button');
    btn.setAttribute('label', this.strings.getString('editbtn'));
    btn.setAttribute('accesskey', this.strings.getString('editbtn_accesskey'));
    btn.setAttribute('id', 'edit');
    btn.setAttribute('disabled', 'true');
    btn.setAttribute('oncommand', 'gEngineManagerDialog.editAlias()');

    var up = document.getElementById('up');
    cvbox.insertBefore(btn, up);

    // It's unlikely another extension will add a second tree but it's better to be safe
    var tree = document.getElementById('engineList');
    var cols = tree.childNodes;
    var col = null;
    for (var i = 0; i < cols.length; i++) {
      if (cols[i].tagName == 'treecols')
        col = cols[i];
    }

    var ncol = document.getElementById('engineName');
    ncol.removeAttribute('hideheader');
    ncol.setAttribute('flex', '3');
    ncol.setAttribute('label', this.strings.getString('namecol'));

    var kcol = document.createElementNS(kXUL_NS, 'treecol');
    kcol.setAttribute('id', 'engineAlias');
    kcol.setAttribute('label', this.strings.getString('keycol'));
    kcol.setAttribute('flex', '1');
    col.appendChild(kcol);

    var osel = tree.getAttribute('onselect');
    tree.setAttribute('onselect', osel + 'schwords.onSelect();');
    tree.setAttribute('ondblclick', 'gEngineManagerDialog.editAlias();');
  },

  onSelect: function() {
    var rm = document.getElementById('remove');
    var edit = document.getElementById('edit');
    edit.disabled = rm.disabled;
  }
};
window.addEventListener("load", function(e) { schwords.init(e); }, false);
